if (Nanika:GetClass() ~= "PALADIN") then return end

-- Globals/Locals
Nanika.Paladin = Nanika:NewModule("Paladin")
Nanika.Paladin.Blessings = Nanika.Paladin:NewModule("Blessings", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0")
local Paladin = Nanika.Paladin
local Blessings = Nanika.Paladin.Blessings
local settings = {}
local defaults = {}

-- Nanika.Paladin
function Paladin:OnInitialize()
	self.db = Nanika.db:RegisterNamespace("Paladin")
end

function Blessings:OnInitialize()
	-- slash commands
	self:RegisterChatCommand("nanikablessings", "OnSlashCommand")
	-- default settings
	defaults.CurrentSpellIndex = {}
	defaults.CurrentTargets = {}
	-- set up db
	local dbsettings = { profile = defaults }
	self.db = Nanika.db:RegisterNamespace("Paladin Blessings")
	self.db:RegisterDefaults(dbsettings)
	settings = self.db.profile

	self.RegisterEvents = {"PLAYER_REGEN_ENABLED","PLAYER_REGEN_DISABLED","PLAYER_SPECIALIZATION_CHANGED","PET_BATTLE_OPENING_START","PET_BATTLE_CLOSE"}
	self.ButtonCount = 3
	self.UI = {}
	self.UI.BaseName = "NanikaPaladinBlessings"
	self.UI.Frame = NanikaPaladinBlessingsFrame
	self.UI.Buttons = {}
	self.Spells = {
		[0] = GetSpellInfo(203528), --GreaterBlessingOfMight
		[1] = GetSpellInfo(203538), --GreaterBlessingOfKings
		[2] = GetSpellInfo(203539), --GreaterBlessingOfWisdom
	}
	self.SpellAbbreviations = { [0] = "M", [1] = "K", [2] = "W", }
	self.SpellCount = 3
end
function Blessings:OnEnable()
	for i, value in ipairs(self.RegisterEvents) do self:RegisterEvent(value) end
	for i = 0, self.ButtonCount - 1 do self:CreateBlessingButton(i) end
	self:Show()
end
function Blessings:OnDisable()
	for i, value in ipairs(self.RegisterEvents) do self:UnregisterEvent(value) end
	self:Hide()
end
function Blessings:Show(fadetime, fadefrom, fadeto)
	if (GetSpecialization() ~= 3) then self:Hide() return end
	if (fadetime) then UIFrameFadeIn(self.UI.Frame, fadetime, fadefrom, fadeto)	end
	if (self.UI.Frame:IsShown()) then return end
	Blessings:OnRefresh()
	self.UI.Frame:Show()
	Blessings:InitializeBlessings()
	self.UI.RefreshTimer = self:ScheduleRepeatingTimer("OnRefresh", 1)
end
function Blessings:Hide()
	self.UI.Frame:Hide()
	if (self.UI.RefreshTimer ~= nil) then self:CancelTimer(self.UI.RefreshTimer) return end
end
function Blessings:CreateBlessingButton(i)
	local name = self.UI.BaseName.."Button"..i
	local basename = "SecureActionButtonTemplate, "..self.UI.BaseName.."ButtonBase"
	local button = CreateFrame("Button", name, self.UI.Frame, basename)
	self.UI.Buttons[i] = button
	button.Index = i
	button:SetPoint("TOPLEFT", self.UI.Frame, "BOTTOMLEFT", 0, (-34 * i))
	button:SetPoint("BOTTOMRIGHT", self.UI.Frame, "BOTTOMRIGHT", 0, -34 + (-34 * i))

	button.IconFrame = _G[name.."IconFrame"]
	button.IconTexture = _G[name.."IconFrameIconTexture"]
	button.TargetString = _G[name.."TargetString"]
	button.ExpirationString = _G[name.."ExpirationString"]
	button:RegisterForClicks("LeftButtonDown", "RightButtonDown")

	-- Left mouse button down (cast blessing)
	button:SetAttribute("type1", "spell")

	-- Right mouse button down (show target selection menu)
	button:SetAttribute("type2", "targetmenu")
	button.targetmenu = function() 
        local menu = CreateFrame("Frame", name.."DropDownMenu", UIParent, "UIDropDownMenuTemplate");
        menu.initialize = function(dropdown, level, menuList) self:InitializeTargetSelectionMenu(button, dropdown, level, menuList) end
        menu.displayMode = "MENU"
        ToggleDropDownMenu(1, nil, menu, "cursor", 10, -8)
	end

	-- Mouse wheel up/down (change blessing)
	button:EnableMouseWheel(true)
	button:SetScript("OnMouseWheel", function(iconframe, delta)
		self:AssignSpell(button, button.CurrentSpellIndex + delta)
	end)
end
function Blessings:OnSlashCommand(input)
	if input == "reset" then
		self.db:ResetProfile()
		ReloadUI()
	return end
end
function Blessings:InitializeBlessings()
	if self.HasInitializedBlessings then return end
	self.HasInitializedBlessings = true
	for i = 0, self.ButtonCount - 1 do
		local button = self.UI.Buttons[i]
		self:AssignSpell(button, settings.CurrentSpellIndex[i] or (i + 1))
		self:AssignTarget(button, UnitName(settings.CurrentTargets[i]) or settings.CurrentTargets[i] or UnitName("player"), settings.CurrentTargets[i] or "player")
	end
end
function Blessings:InitializeTargetSelectionMenu(button, dropdown, level, menuList)
	local info
	if level == 1 then
		info = UIDropDownMenu_CreateInfo()
		info.isTitle = 1
		info.text = "Select target"
		UIDropDownMenu_AddButton(info, 1)

		pname = UnitName("player")
		self:AddTargetSelectionOption(button, pname, pname, "player")

		tname, trealm = UnitName("target")
		if tname then
			if trealm and trealm ~= "" then tname = tname.."-"..trealm end
			self:AddTargetSelectionOption(button, "Target: "..tname, tname)
		end

		fname, frealm = UnitName("focus")
		if fname then
			if frealm and frealm ~= "" then fname = fname.."-"..frealm end
			self:AddTargetSelectionOption(button, "Focus: "..fname, fname)
		end

		raid = Nanika:GetRaidLayout()
		if raid then
			info = UIDropDownMenu_CreateInfo()
			UIDropDownMenu_AddSeparator(info, 1)
			for i = 1, MAX_RAID_GROUPS do
				if raid[i].MemberCount > 0 then
					info = UIDropDownMenu_CreateInfo()
					info.text = "Party "..i
					info.notCheckable = true
					info.hasArrow = true
					info.menuList = raid[i]
					UIDropDownMenu_AddButton(info, 1)
				end
			end
			info = UIDropDownMenu_CreateInfo()
			UIDropDownMenu_AddSeparator(info, 1)
			if raid["TANK"].MemberCount > 0 then
				info = UIDropDownMenu_CreateInfo()
				info.text = "Tank"
				info.notCheckable = true
				info.hasArrow = true
				info.menuList = raid["TANK"]
				UIDropDownMenu_AddButton(info, 1)
			end
			if raid["HEALER"].MemberCount > 0 then
				info = UIDropDownMenu_CreateInfo()
				info.text = "Healer"
				info.notCheckable = true
				info.hasArrow = true
				info.menuList = raid["HEALER"]
				UIDropDownMenu_AddButton(info, 1)
			end
			if raid["DAMAGER"].MemberCount > 0 then
				info = UIDropDownMenu_CreateInfo()
				info.text = "Dps"
				info.notCheckable = true
				info.hasArrow = true
				info.menuList = raid["DAMAGER"]
				UIDropDownMenu_AddButton(info, 1)
			end
		end
	elseif menuList then
		for i = 1, menuList.MemberCount do
			local member = menuList.Members[i]
			--if we assign a party/raid id then we risk the chance of it not being accurate when casting
			self:AddTargetSelectionOption(button, member.Name, member.Name, member.Name, 2)
		end
	end
end
function Blessings:AddTargetSelectionOption(button, labelname, targetname, targetunitid, level)
	targetname = targetname or labelname
	targetunitid = targetunitid or targetname
	for i = 1, self.SpellCount do
		local spellName = self.Spells[i % self.SpellCount]
		local _,_,_,_,_,_,buffExpiration,caster = UnitBuff(targetunitid, spellName)
		if buffExpiration and caster ~= "player" then
			labelname = labelname .. ", " .. self.SpellAbbreviations[i % self.SpellCount]
		end
	end
	info = UIDropDownMenu_CreateInfo()
	info.button = button
	info.text = labelname
	info.notCheckable = true
	if UnitIsPlayer(targetunitid) then
		info.func = function() self:AssignTarget(button, targetname, targetunitid) end
	else
		info.disabled = true
	end
	UIDropDownMenu_AddButton(info, level or 1)
end

function Blessings:AssignSpell(button, i)
	i = (i + self.SpellCount) % self.SpellCount
	settings.CurrentSpellIndex[button.Index] = i
	button.CurrentSpellIndex = i
	local name,_,texture = GetSpellInfo(self.Spells[i])
	button:SetAttribute("spell1", name)
	button.IconTexture:SetTexture(texture)
	self:RefreshButton(button)
end
function Blessings:AssignTarget(button, name, unitid)
	settings.CurrentTargets[button.Index] = unitid
	button.TargetString:SetText(name)
	button:SetAttribute("unit1", unitid)
end

function Blessings:OnRefresh()
	for i = 0, self.ButtonCount - 1 do self:RefreshButton(self.UI.Buttons[i]) end
end
function Blessings:RefreshButton(button)
	local unitid = button:GetAttribute("unit1")
	local spellName = self.Spells[button.CurrentSpellIndex]
	if unitid and spellName then
		local _,_,_,_,_,buffDuration,buffExpiration,caster = UnitBuff(unitid, spellName)
		if buffExpiration then
			local expires = max(0, buffExpiration - GetTime())
			local mins = floor(expires / 60)
			local secs = floor(expires % 60)
			local part = expires / buffDuration
			if caster == "player" then
				button.ExpirationString:SetText(string.format("%d:%02d", mins, secs))
			else if caster == nil then
				button.ExpirationString:SetText(string.format("%d:%02d (Unknown caster)", mins, secs))
			else
				button.ExpirationString:SetText(string.format("%d:%02d (%s)", mins, secs, UnitName(caster)))
			end end
			if part > 0.5 then
				button:SetBackdropColor((1 - part) * 2, 1, 0)
			else
				button:SetBackdropColor(1, part * 2, 0)
			end
		else
			button.ExpirationString:SetText("-")
			button:SetBackdropColor(1, 0, 0)
		end
	end
end
-- Events
function Blessings:PLAYER_REGEN_ENABLED() self:Show(1, .65, 1) end
function Blessings:PLAYER_REGEN_DISABLED() self:Show(.5, 1, .65) end
function Blessings:PLAYER_SPECIALIZATION_CHANGED() self:Show() end
function Blessings:PET_BATTLE_OPENING_START() self:Hide() end
function Blessings:PET_BATTLE_CLOSE() self:Show() end